@@ -11,155 +11,12 @@
 
 #include <arm_neon.h>
 
-#include "config/aom_dsp_rtcd.h"
-#include "config/aom_config.h"
+#include "aom/aom_integer.h"
 #include "aom_dsp/arm/mem_neon.h"
 #include "aom_dsp/arm/sum_neon.h"
-#include "aom/aom_integer.h"
 #include "aom_ports/mem.h"
-
-#if defined(__ARM_FEATURE_DOTPROD)
-
-static INLINE void variance_4xh_neon(const uint8_t *src, int src_stride,
-                                     const uint8_t *ref, int ref_stride, int h,
-                                     uint32_t *sse, int *sum) {
-  uint32x4_t src_sum = vdupq_n_u32(0);
-  uint32x4_t ref_sum = vdupq_n_u32(0);
-  uint32x4_t sse_u32 = vdupq_n_u32(0);
-
-  int i = 0;
-  do {
-    uint8x16_t s = load_unaligned_u8q(src, src_stride);
-    uint8x16_t r = load_unaligned_u8q(ref, ref_stride);
-
-    src_sum = vdotq_u32(src_sum, s, vdupq_n_u8(1));
-    ref_sum = vdotq_u32(ref_sum, r, vdupq_n_u8(1));
-
-    uint8x16_t abs_diff = vabdq_u8(s, r);
-    sse_u32 = vdotq_u32(sse_u32, abs_diff, abs_diff);
-
-    src += 4 * src_stride;
-    ref += 4 * ref_stride;
-    i += 4;
-  } while (i < h);
-
-  int32x4_t sum_diff =
-      vsubq_s32(vreinterpretq_s32_u32(src_sum), vreinterpretq_s32_u32(ref_sum));
-  *sum = horizontal_add_s32x4(sum_diff);
-  *sse = horizontal_add_u32x4(sse_u32);
-}
-
-static INLINE void variance_8xh_neon(const uint8_t *src, int src_stride,
-                                     const uint8_t *ref, int ref_stride, int h,
-                                     uint32_t *sse, int *sum) {
-  uint32x4_t src_sum = vdupq_n_u32(0);
-  uint32x4_t ref_sum = vdupq_n_u32(0);
-  uint32x4_t sse_u32 = vdupq_n_u32(0);
-
-  int i = 0;
-  do {
-    uint8x16_t s = vcombine_u8(vld1_u8(src), vld1_u8(src + src_stride));
-    uint8x16_t r = vcombine_u8(vld1_u8(ref), vld1_u8(ref + ref_stride));
-
-    src_sum = vdotq_u32(src_sum, s, vdupq_n_u8(1));
-    ref_sum = vdotq_u32(ref_sum, r, vdupq_n_u8(1));
-
-    uint8x16_t abs_diff = vabdq_u8(s, r);
-    sse_u32 = vdotq_u32(sse_u32, abs_diff, abs_diff);
-
-    src += 2 * src_stride;
-    ref += 2 * ref_stride;
-    i += 2;
-  } while (i < h);
-
-  int32x4_t sum_diff =
-      vsubq_s32(vreinterpretq_s32_u32(src_sum), vreinterpretq_s32_u32(ref_sum));
-  *sum = horizontal_add_s32x4(sum_diff);
-  *sse = horizontal_add_u32x4(sse_u32);
-}
-
-static INLINE void variance_16xh_neon(const uint8_t *src, int src_stride,
-                                      const uint8_t *ref, int ref_stride, int h,
-                                      uint32_t *sse, int *sum) {
-  uint32x4_t src_sum = vdupq_n_u32(0);
-  uint32x4_t ref_sum = vdupq_n_u32(0);
-  uint32x4_t sse_u32 = vdupq_n_u32(0);
-
-  int i = 0;
-  do {
-    uint8x16_t s = vld1q_u8(src);
-    uint8x16_t r = vld1q_u8(ref);
-
-    src_sum = vdotq_u32(src_sum, s, vdupq_n_u8(1));
-    ref_sum = vdotq_u32(ref_sum, r, vdupq_n_u8(1));
-
-    uint8x16_t abs_diff = vabdq_u8(s, r);
-    sse_u32 = vdotq_u32(sse_u32, abs_diff, abs_diff);
-
-    src += src_stride;
-    ref += ref_stride;
-    i++;
-  } while (i < h);
-
-  int32x4_t sum_diff =
-      vsubq_s32(vreinterpretq_s32_u32(src_sum), vreinterpretq_s32_u32(ref_sum));
-  *sum = horizontal_add_s32x4(sum_diff);
-  *sse = horizontal_add_u32x4(sse_u32);
-}
-
-static INLINE void variance_large_neon(const uint8_t *src, int src_stride,
-                                       const uint8_t *ref, int ref_stride,
-                                       int w, int h, uint32_t *sse, int *sum) {
-  uint32x4_t src_sum = vdupq_n_u32(0);
-  uint32x4_t ref_sum = vdupq_n_u32(0);
-  uint32x4_t sse_u32 = vdupq_n_u32(0);
-
-  int i = 0;
-  do {
-    int j = 0;
-    do {
-      uint8x16_t s = vld1q_u8(src + j);
-      uint8x16_t r = vld1q_u8(ref + j);
-
-      src_sum = vdotq_u32(src_sum, s, vdupq_n_u8(1));
-      ref_sum = vdotq_u32(ref_sum, r, vdupq_n_u8(1));
-
-      uint8x16_t abs_diff = vabdq_u8(s, r);
-      sse_u32 = vdotq_u32(sse_u32, abs_diff, abs_diff);
-
-      j += 16;
-    } while (j < w);
-
-    src += src_stride;
-    ref += ref_stride;
-    i++;
-  } while (i < h);
-
-  int32x4_t sum_diff =
-      vsubq_s32(vreinterpretq_s32_u32(src_sum), vreinterpretq_s32_u32(ref_sum));
-  *sum = horizontal_add_s32x4(sum_diff);
-  *sse = horizontal_add_u32x4(sse_u32);
-}
-
-static INLINE void variance_32xh_neon(const uint8_t *src, int src_stride,
-                                      const uint8_t *ref, int ref_stride, int h,
-                                      uint32_t *sse, int *sum) {
-  variance_large_neon(src, src_stride, ref, ref_stride, 32, h, sse, sum);
-}
-
-static INLINE void variance_64xh_neon(const uint8_t *src, int src_stride,
-                                      const uint8_t *ref, int ref_stride, int h,
-                                      uint32_t *sse, int *sum) {
-  variance_large_neon(src, src_stride, ref, ref_stride, 64, h, sse, sum);
-}
-
-static INLINE void variance_128xh_neon(const uint8_t *src, int src_stride,
-                                       const uint8_t *ref, int ref_stride,
-                                       int h, uint32_t *sse, int *sum) {
-  variance_large_neon(src, src_stride, ref, ref_stride, 128, h, sse, sum);
-}
-
-#else  // !defined(__ARM_FEATURE_DOTPROD)
+#include "config/aom_config.h"
+#include "config/aom_dsp_rtcd.h"
 
 static INLINE void variance_4xh_neon(const uint8_t *src, int src_stride,
                                      const uint8_t *ref, int ref_stride, int h,
@@ -171,7 +28,7 @@ static INLINE void variance_4xh_neon(const uint8_t *src, int src_stride,
   // 32767 / 255 ~= 128, but we use an 8-wide accumulator; so 256 4-wide rows.
   assert(h <= 256);
 
-  int i = 0;
+  int i = h;
   do {
     uint8x8_t s = load_unaligned_u8(src, src_stride);
     uint8x8_t r = load_unaligned_u8(ref, ref_stride);
@@ -184,8 +41,8 @@ static INLINE void variance_4xh_neon(const uint8_t *src, int src_stride,
 
     src += 2 * src_stride;
     ref += 2 * ref_stride;
-    i += 2;
-  } while (i < h);
+    i -= 2;
+  } while (i != 0);
 
   *sum = horizontal_add_s16x8(sum_s16);
   *sse = (uint32_t)horizontal_add_s32x4(sse_s32);
@@ -201,7 +58,7 @@ static INLINE void variance_8xh_neon(const uint8_t *src, int src_stride,
   // 32767 / 255 ~= 128
   assert(h <= 128);
 
-  int i = 0;
+  int i = h;
   do {
     uint8x8_t s = vld1_u8(src);
     uint8x8_t r = vld1_u8(ref);
@@ -215,8 +72,7 @@ static INLINE void variance_8xh_neon(const uint8_t *src, int src_stride,
 
     src += src_stride;
     ref += ref_stride;
-    i++;
-  } while (i < h);
+  } while (--i != 0);
 
   *sum = horizontal_add_s16x8(sum_s16);
   *sse = (uint32_t)horizontal_add_s32x4(vaddq_s32(sse_s32[0], sse_s32[1]));
@@ -232,7 +88,7 @@ static INLINE void variance_16xh_neon(const uint8_t *src, int src_stride,
   // 32767 / 255 ~= 128, so 128 16-wide rows.
   assert(h <= 128);
 
-  int i = 0;
+  int i = h;
   do {
     uint8x16_t s = vld1q_u8(src);
     uint8x16_t r = vld1q_u8(ref);
@@ -256,8 +112,7 @@ static INLINE void variance_16xh_neon(const uint8_t *src, int src_stride,
 
     src += src_stride;
     ref += ref_stride;
-    i++;
-  } while (i < h);
+  } while (--i != 0);
 
   *sum = horizontal_add_s16x8(vaddq_s16(sum_s16[0], sum_s16[1]));
   *sse = (uint32_t)horizontal_add_s32x4(vaddq_s32(sse_s32[0], sse_s32[1]));
@@ -337,8 +192,6 @@ static INLINE void variance_128xh_neon(const uint8_t *src, int src_stride,
   variance_large_neon(src, src_stride, ref, ref_stride, 128, h, 16, sse, sum);
 }
 
-#endif  // defined(__ARM_FEATURE_DOTPROD)
-
 #define VARIANCE_WXH_NEON(w, h, shift)                                        \
   unsigned int aom_variance##w##x##h##_neon(                                  \
       const uint8_t *src, int src_stride, const uint8_t *ref, int ref_stride, \
@@ -378,17 +231,6 @@ VARIANCE_WXH_NEON(128, 128, 14)
 
 #undef VARIANCE_WXH_NEON
 
-void aom_get8x8var_neon(const uint8_t *src, int src_stride, const uint8_t *ref,
-                        int ref_stride, unsigned int *sse, int *sum) {
-  variance_8xh_neon(src, src_stride, ref, ref_stride, 8, sse, sum);
-}
-
-void aom_get16x16var_neon(const uint8_t *src, int src_stride,
-                          const uint8_t *ref, int ref_stride, unsigned int *sse,
-                          int *sum) {
-  variance_16xh_neon(src, src_stride, ref, ref_stride, 16, sse, sum);
-}
-
 // TODO(yunqingwang): Perform variance of two/four 8x8 blocks similar to that of
 // AVX2. Also, implement the NEON for variance computation present in this
 // function.
@@ -397,7 +239,7 @@ void aom_get_var_sse_sum_8x8_quad_neon(const uint8_t *src, int src_stride,
                                        uint32_t *sse8x8, int *sum8x8,
                                        unsigned int *tot_sse, int *tot_sum,
                                        uint32_t *var8x8) {
-  // Loop over 4 8x8 blocks. Process one 8x32 block.
+  // Loop over four 8x8 blocks. Process one 8x32 block.
   for (int k = 0; k < 4; k++) {
     variance_8xh_neon(src + (k * 8), src_stride, ref + (k * 8), ref_stride, 8,
                       &sse8x8[k], &sum8x8[k]);
@@ -405,76 +247,31 @@ void aom_get_var_sse_sum_8x8_quad_neon(const uint8_t *src, int src_stride,
 
   *tot_sse += sse8x8[0] + sse8x8[1] + sse8x8[2] + sse8x8[3];
   *tot_sum += sum8x8[0] + sum8x8[1] + sum8x8[2] + sum8x8[3];
-  for (int i = 0; i < 4; i++)
+  for (int i = 0; i < 4; i++) {
     var8x8[i] = sse8x8[i] - (uint32_t)(((int64_t)sum8x8[i] * sum8x8[i]) >> 6);
+  }
 }
 
-#if defined(__ARM_FEATURE_DOTPROD)
+void aom_get_var_sse_sum_16x16_dual_neon(const uint8_t *src, int src_stride,
+                                         const uint8_t *ref, int ref_stride,
+                                         uint32_t *sse16x16,
+                                         unsigned int *tot_sse, int *tot_sum,
+                                         uint32_t *var16x16) {
+  int sum16x16[2] = { 0 };
+  // Loop over two 16x16 blocks. Process one 16x32 block.
+  for (int k = 0; k < 2; k++) {
+    variance_16xh_neon(src + (k * 16), src_stride, ref + (k * 16), ref_stride,
+                       16, &sse16x16[k], &sum16x16[k]);
+  }
 
-static INLINE unsigned int mse8xh_neon(const uint8_t *src, int src_stride,
-                                       const uint8_t *ref, int ref_stride,
-                                       unsigned int *sse, int h) {
-  uint32x4_t sse_u32 = vdupq_n_u32(0);
-
-  int i = 0;
-  do {
-    uint8x16_t s = vcombine_u8(vld1_u8(src), vld1_u8(src + src_stride));
-    uint8x16_t r = vcombine_u8(vld1_u8(ref), vld1_u8(ref + ref_stride));
-
-    uint8x16_t abs_diff = vabdq_u8(s, r);
-
-    sse_u32 = vdotq_u32(sse_u32, abs_diff, abs_diff);
-
-    src += 2 * src_stride;
-    ref += 2 * ref_stride;
-    i += 2;
-  } while (i < h);
-
-  *sse = horizontal_add_u32x4(sse_u32);
-  return horizontal_add_u32x4(sse_u32);
+  *tot_sse += sse16x16[0] + sse16x16[1];
+  *tot_sum += sum16x16[0] + sum16x16[1];
+  for (int i = 0; i < 2; i++) {
+    var16x16[i] =
+        sse16x16[i] - (uint32_t)(((int64_t)sum16x16[i] * sum16x16[i]) >> 8);
+  }
 }
 
-static INLINE unsigned int mse16xh_neon(const uint8_t *src, int src_stride,
-                                        const uint8_t *ref, int ref_stride,
-                                        unsigned int *sse, int h) {
-  uint32x4_t sse_u32[2] = { vdupq_n_u32(0), vdupq_n_u32(0) };
-
-  int i = 0;
-  do {
-    uint8x16_t s0 = vld1q_u8(src);
-    uint8x16_t s1 = vld1q_u8(src + src_stride);
-    uint8x16_t r0 = vld1q_u8(ref);
-    uint8x16_t r1 = vld1q_u8(ref + ref_stride);
-
-    uint8x16_t abs_diff0 = vabdq_u8(s0, r0);
-    uint8x16_t abs_diff1 = vabdq_u8(s1, r1);
-
-    sse_u32[0] = vdotq_u32(sse_u32[0], abs_diff0, abs_diff0);
-    sse_u32[1] = vdotq_u32(sse_u32[1], abs_diff1, abs_diff1);
-
-    src += 2 * src_stride;
-    ref += 2 * ref_stride;
-    i += 2;
-  } while (i < h);
-
-  *sse = horizontal_add_u32x4(vaddq_u32(sse_u32[0], sse_u32[1]));
-  return horizontal_add_u32x4(vaddq_u32(sse_u32[0], sse_u32[1]));
-}
-
-unsigned int aom_get4x4sse_cs_neon(const uint8_t *src, int src_stride,
-                                   const uint8_t *ref, int ref_stride) {
-  uint8x16_t s = load_unaligned_u8q(src, src_stride);
-  uint8x16_t r = load_unaligned_u8q(ref, ref_stride);
-
-  uint8x16_t abs_diff = vabdq_u8(s, r);
-
-  uint32x4_t sse = vdotq_u32(vdupq_n_u32(0), abs_diff, abs_diff);
-
-  return horizontal_add_u32x4(sse);
-}
-
-#else  // !defined(__ARM_FEATURE_DOTPROD)
-
 static INLINE unsigned int mse8xh_neon(const uint8_t *src, int src_stride,
                                        const uint8_t *ref, int ref_stride,
                                        unsigned int *sse, int h) {
@@ -483,7 +280,7 @@ static INLINE unsigned int mse8xh_neon(const uint8_t *src, int src_stride,
   uint16x8_t diff[2];
   int32x4_t sse_s32[2] = { vdupq_n_s32(0), vdupq_n_s32(0) };
 
-  int i = 0;
+  int i = h;
   do {
     s[0] = vld1_u8(src);
     src += src_stride;
@@ -507,8 +304,8 @@ static INLINE unsigned int mse8xh_neon(const uint8_t *src, int src_stride,
     sse_s32[0] = vmlal_s16(sse_s32[0], diff_hi[0], diff_hi[0]);
     sse_s32[1] = vmlal_s16(sse_s32[1], diff_hi[1], diff_hi[1]);
 
-    i += 2;
-  } while (i < h);
+    i -= 2;
+  } while (i != 0);
 
   sse_s32[0] = vaddq_s32(sse_s32[0], sse_s32[1]);
 
@@ -525,7 +322,7 @@ static INLINE unsigned int mse16xh_neon(const uint8_t *src, int src_stride,
   int32x4_t sse_s32[4] = { vdupq_n_s32(0), vdupq_n_s32(0), vdupq_n_s32(0),
                            vdupq_n_s32(0) };
 
-  int i = 0;
+  int i = h;
   do {
     s[0] = vld1q_u8(src);
     src += src_stride;
@@ -561,8 +358,8 @@ static INLINE unsigned int mse16xh_neon(const uint8_t *src, int src_stride,
     sse_s32[2] = vmlal_s16(sse_s32[2], diff_hi[2], diff_hi[2]);
     sse_s32[3] = vmlal_s16(sse_s32[3], diff_hi[3], diff_hi[3]);
 
-    i += 2;
-  } while (i < h);
+    i -= 2;
+  } while (i != 0);
 
   sse_s32[0] = vaddq_s32(sse_s32[0], sse_s32[1]);
   sse_s32[2] = vaddq_s32(sse_s32[2], sse_s32[3]);
@@ -572,42 +369,6 @@ static INLINE unsigned int mse16xh_neon(const uint8_t *src, int src_stride,
   return horizontal_add_u32x4(vreinterpretq_u32_s32(sse_s32[0]));
 }
 
-unsigned int aom_get4x4sse_cs_neon(const uint8_t *src, int src_stride,
-                                   const uint8_t *ref, int ref_stride) {
-  uint8x8_t s[4], r[4];
-  int16x4_t diff[4];
-  int32x4_t sse;
-
-  s[0] = vld1_u8(src);
-  src += src_stride;
-  r[0] = vld1_u8(ref);
-  ref += ref_stride;
-  s[1] = vld1_u8(src);
-  src += src_stride;
-  r[1] = vld1_u8(ref);
-  ref += ref_stride;
-  s[2] = vld1_u8(src);
-  src += src_stride;
-  r[2] = vld1_u8(ref);
-  ref += ref_stride;
-  s[3] = vld1_u8(src);
-  r[3] = vld1_u8(ref);
-
-  diff[0] = vget_low_s16(vreinterpretq_s16_u16(vsubl_u8(s[0], r[0])));
-  diff[1] = vget_low_s16(vreinterpretq_s16_u16(vsubl_u8(s[1], r[1])));
-  diff[2] = vget_low_s16(vreinterpretq_s16_u16(vsubl_u8(s[2], r[2])));
-  diff[3] = vget_low_s16(vreinterpretq_s16_u16(vsubl_u8(s[3], r[3])));
-
-  sse = vmull_s16(diff[0], diff[0]);
-  sse = vmlal_s16(sse, diff[1], diff[1]);
-  sse = vmlal_s16(sse, diff[2], diff[2]);
-  sse = vmlal_s16(sse, diff[3], diff[3]);
-
-  return horizontal_add_u32x4(vreinterpretq_u32_s32(sse));
-}
-
-#endif  // defined(__ARM_FEATURE_DOTPROD)
-
 #define MSE_WXH_NEON(w, h)                                                 \
   unsigned int aom_mse##w##x##h##_neon(const uint8_t *src, int src_stride, \
                                        const uint8_t *ref, int ref_stride, \
@@ -623,96 +384,87 @@ MSE_WXH_NEON(16, 16)
 
 #undef MSE_WXH_NEON
 
-#define COMPUTE_MSE_16BIT(src_16x8, dst_16x8)                           \
-  /* r7 r6 r5 r4 r3 r2 r1 r0 - 16 bit */                                \
-  const uint16x8_t diff = vabdq_u16(src_16x8, dst_16x8);                \
-  /*r3 r2 r1 r0 - 16 bit */                                             \
-  const uint16x4_t res0_low_16x4 = vget_low_u16(diff);                  \
-  /*r7 r6 r5 r4 - 16 bit */                                             \
-  const uint16x4_t res0_high_16x4 = vget_high_u16(diff);                \
-  /* (r3*r3)= b3 (r2*r2)= b2 (r1*r1)= b1 (r0*r0)= b0 - 32 bit */        \
-  const uint32x4_t res0_32x4 = vmull_u16(res0_low_16x4, res0_low_16x4); \
-  /* (r7*r7)= b7 (r6*r6)= b6 (r5*r5)= b5 (r4*r4)= b4 - 32 bit*/         \
-  /* b3+b7 b2+b6 b1+b5 b0+b4 - 32 bit*/                                 \
-  const uint32x4_t res_32x4 =                                           \
-      vmlal_u16(res0_32x4, res0_high_16x4, res0_high_16x4);             \
-                                                                        \
-  /*a1 a0 - 64 bit*/                                                    \
-  const uint64x2_t vl = vpaddlq_u32(res_32x4);                          \
-  /*a1+a2= f1 a3+a0= f0*/                                               \
-  square_result = vaddq_u64(square_result, vl);
+static INLINE uint64x2_t mse_accumulate_u16_u8_8x2(uint64x2_t sum,
+                                                   uint16x8_t s0, uint16x8_t s1,
+                                                   uint8x8_t d0, uint8x8_t d1) {
+  int16x8_t e0 = vreinterpretq_s16_u16(vsubw_u8(s0, d0));
+  int16x8_t e1 = vreinterpretq_s16_u16(vsubw_u8(s1, d1));
 
-static AOM_INLINE uint64_t mse_4xh_16bit_neon(uint8_t *dst, int dstride,
-                                              uint16_t *src, int sstride,
-                                              int h) {
-  uint64x2_t square_result = vdupq_n_u64(0);
-  uint32_t d0, d1;
-  int i = 0;
-  uint8_t *dst_ptr = dst;
-  uint16_t *src_ptr = src;
-  do {
-    // d03 d02 d01 d00 - 8 bit
-    memcpy(&d0, dst_ptr, 4);
-    dst_ptr += dstride;
-    // d13 d12 d11 d10 - 8 bit
-    memcpy(&d1, dst_ptr, 4);
-    dst_ptr += dstride;
-    // duplication
-    uint8x8_t tmp0_8x8 = vreinterpret_u8_u32(vdup_n_u32(d0));
-    // d03 d02 d01 d00 - 16 bit
-    const uint16x4_t dst0_16x4 = vget_low_u16(vmovl_u8(tmp0_8x8));
-    // duplication
-    tmp0_8x8 = vreinterpret_u8_u32(vdup_n_u32(d1));
-    // d13 d12 d11 d10 - 16 bit
-    const uint16x4_t dst1_16x4 = vget_low_u16(vmovl_u8(tmp0_8x8));
-    // d13 d12 d11 d10 d03 d02 d01 d00 - 16 bit
-    const uint16x8_t dst_16x8 = vcombine_u16(dst0_16x4, dst1_16x4);
+  int32x4_t mse = vmull_s16(vget_low_s16(e0), vget_low_s16(e0));
+  mse = vmlal_s16(mse, vget_high_s16(e0), vget_high_s16(e0));
+  mse = vmlal_s16(mse, vget_low_s16(e1), vget_low_s16(e1));
+  mse = vmlal_s16(mse, vget_high_s16(e1), vget_high_s16(e1));
 
-    // b1r0 - s03 s02 s01 s00 - 16 bit
-    const uint16x4_t src0_16x4 = vld1_u16(src_ptr);
-    src_ptr += sstride;
-    // b1r1 - s13 s12 s11 s10 - 16 bit
-    const uint16x4_t src1_16x4 = vld1_u16(src_ptr);
-    src_ptr += sstride;
-    // s13 s12 s11 s10 s03 s02 s01 s00 - 16 bit
-    const uint16x8_t src_16x8 = vcombine_u16(src0_16x4, src1_16x4);
-
-    COMPUTE_MSE_16BIT(src_16x8, dst_16x8)
-    i += 2;
-  } while (i < h);
-  uint64x1_t sum =
-      vadd_u64(vget_high_u64(square_result), vget_low_u64(square_result));
-  return vget_lane_u64(sum, 0);
+  return vpadalq_u32(sum, vreinterpretq_u32_s32(mse));
 }
 
-static AOM_INLINE uint64_t mse_8xh_16bit_neon(uint8_t *dst, int dstride,
-                                              uint16_t *src, int sstride,
-                                              int h) {
-  uint64x2_t square_result = vdupq_n_u64(0);
-  int i = 0;
-  do {
-    // d7 d6 d5 d4 d3 d2 d1 d0 - 8 bit
-    const uint16x8_t dst_16x8 = vmovl_u8(vld1_u8(&dst[i * dstride]));
-    // s7 s6 s5 s4 s3 s2 s1 s0 - 16 bit
-    const uint16x8_t src_16x8 = vld1q_u16(&src[i * sstride]);
+static uint64x2_t mse_wxh_16bit(uint8_t *dst, int dstride, const uint16_t *src,
+                                int sstride, int w, int h) {
+  assert((w == 8 || w == 4) && (h == 8 || h == 4));
 
-    COMPUTE_MSE_16BIT(src_16x8, dst_16x8)
-    i++;
-  } while (i < h);
-  uint64x1_t sum =
-      vadd_u64(vget_high_u64(square_result), vget_low_u64(square_result));
-  return vget_lane_u64(sum, 0);
+  uint64x2_t sum = vdupq_n_u64(0);
+
+  if (w == 8) {
+    do {
+      uint8x8_t d0 = vld1_u8(dst + 0 * dstride);
+      uint8x8_t d1 = vld1_u8(dst + 1 * dstride);
+      uint16x8_t s0 = vld1q_u16(src + 0 * sstride);
+      uint16x8_t s1 = vld1q_u16(src + 1 * sstride);
+
+      sum = mse_accumulate_u16_u8_8x2(sum, s0, s1, d0, d1);
+
+      dst += 2 * dstride;
+      src += 2 * sstride;
+      h -= 2;
+    } while (h != 0);
+  } else {
+    do {
+      uint8x8_t d0 = load_unaligned_u8_4x2(dst + 0 * dstride, dstride);
+      uint8x8_t d1 = load_unaligned_u8_4x2(dst + 2 * dstride, dstride);
+      uint16x8_t s0 = load_unaligned_u16_4x2(src + 0 * sstride, sstride);
+      uint16x8_t s1 = load_unaligned_u16_4x2(src + 2 * sstride, sstride);
+
+      sum = mse_accumulate_u16_u8_8x2(sum, s0, s1, d0, d1);
+
+      dst += 4 * dstride;
+      src += 4 * sstride;
+      h -= 4;
+    } while (h != 0);
+  }
+
+  return sum;
 }
 
 // Computes mse for a given block size. This function gets called for specific
 // block sizes, which are 8x8, 8x4, 4x8 and 4x4.
 uint64_t aom_mse_wxh_16bit_neon(uint8_t *dst, int dstride, uint16_t *src,
                                 int sstride, int w, int h) {
-  assert((w == 8 || w == 4) && (h == 8 || h == 4) &&
-         "w=8/4 and h=8/4 must satisfy");
-  switch (w) {
-    case 4: return mse_4xh_16bit_neon(dst, dstride, src, sstride, h);
-    case 8: return mse_8xh_16bit_neon(dst, dstride, src, sstride, h);
-    default: assert(0 && "unsupported width"); return -1;
+  return horizontal_add_u64x2(mse_wxh_16bit(dst, dstride, src, sstride, w, h));
+}
+
+uint32_t aom_get_mb_ss_neon(const int16_t *a) {
+  int32x4_t sse[2] = { vdupq_n_s32(0), vdupq_n_s32(0) };
+
+  for (int i = 0; i < 256; i = i + 8) {
+    int16x8_t a_s16 = vld1q_s16(a + i);
+
+    sse[0] = vmlal_s16(sse[0], vget_low_s16(a_s16), vget_low_s16(a_s16));
+    sse[1] = vmlal_s16(sse[1], vget_high_s16(a_s16), vget_high_s16(a_s16));
   }
+
+  return horizontal_add_s32x4(vaddq_s32(sse[0], sse[1]));
+}
+
+uint64_t aom_mse_16xh_16bit_neon(uint8_t *dst, int dstride, uint16_t *src,
+                                 int w, int h) {
+  uint64x2_t sum = vdupq_n_u64(0);
+
+  int num_blks = 16 / w;
+  do {
+    sum = vaddq_u64(sum, mse_wxh_16bit(dst, dstride, src, w, w, h));
+    dst += w;
+    src += w * h;
+  } while (--num_blks != 0);
+
+  return horizontal_add_u64x2(sum);
 }
