@@ -729,14 +729,16 @@ VAR_FN(32, 16, 16, 9)
 VAR_FN(16, 32, 16, 9)
 VAR_FN(16, 16, 16, 8)
 VAR_FN(16, 8, 8, 7)
-VAR_FN(16, 4, 16, 6)
-VAR_FN(8, 32, 8, 8)
-VAR_FN(32, 8, 8, 8)
-VAR_FN(16, 64, 16, 10)
-VAR_FN(64, 16, 16, 10)
 VAR_FN(8, 16, 8, 7)
 VAR_FN(8, 8, 8, 6)
 
+#if !CONFIG_REALTIME_ONLY
+VAR_FN(16, 64, 16, 10)
+VAR_FN(32, 8, 8, 8)
+VAR_FN(64, 16, 16, 10)
+VAR_FN(8, 32, 8, 8)
+#endif  // !CONFIG_REALTIME_ONLY
+
 #undef VAR_FN
 
 #define SSE2_HEIGHT(H)                                                 \
