@@ -80,10 +80,3 @@ void aom_free(void *memblk) {
     free(addr);
   }
 }
-
-void *aom_memset16(void *dest, int val, size_t length) {
-  size_t i;
-  uint16_t *dest16 = (uint16_t *)dest;
-  for (i = 0; i < length; i++) *dest16++ = val;
-  return dest;
-}
