@@ -399,13 +399,11 @@ void av1_setup_frame_size(AV1_COMP *cpi) {
 void av1_superres_post_encode(AV1_COMP *cpi) {
   AV1_COMMON *cm = &cpi->common;
 
-  if (!av1_superres_scaled(cm)) return;
-
   assert(cpi->oxcf.superres_cfg.enable_superres);
   assert(!is_lossless_requested(&cpi->oxcf.rc_cfg));
   assert(!cm->features.all_lossless);
 
-  av1_superres_upscale(cm, NULL);
+  av1_superres_upscale(cm, NULL, cpi->image_pyramid_levels);
 
   // If regular resizing is occurring the source will need to be downscaled to
   // match the upscaled superres resolution. Otherwise the original source is
