@@ -33,12 +33,12 @@ typedef std::tuple<const HorverFunc> HorverTestParam;
 
 class HorverTest : public ::testing::TestWithParam<HorverTestParam> {
  public:
-  virtual void SetUp() {
+  void SetUp() override {
     data_buf_ = (int16_t *)aom_malloc(MAX_SB_SQUARE * sizeof(int16_t));
     ASSERT_NE(data_buf_, nullptr);
     target_func_ = GET_PARAM(0);
   }
-  virtual void TearDown() { aom_free(data_buf_); }
+  void TearDown() override { aom_free(data_buf_); }
   void RunHorverTest();
   void RunHorverTest_ExtremeValues();
   void RunHorverSpeedTest(int run_times);
