@@ -48,7 +48,7 @@ class SharpnessTest
         cpu_used_(GET_PARAM(2)), sharpness_level_(GET_PARAM(3)), psnr_(0.0),
         nframes_(0) {}
 
-  ~SharpnessTest() override {}
+  ~SharpnessTest() override = default;
 
   void SetUp() override {
     InitializeConfig(encoding_mode_);
@@ -97,7 +97,7 @@ class SharpnessTest
 
     std::unique_ptr<libaom_test::VideoSource> video(
         new libaom_test::Y4mVideoSource("paris_352_288_30.y4m", 0, kFrames));
-    ASSERT_TRUE(video.get() != NULL);
+    ASSERT_NE(video, nullptr);
 
     ASSERT_NO_FATAL_FAILURE(RunLoop(video.get()));
     const double psnr = GetAveragePsnr();
